#include <iostream>
#include "Bit_stream.h"
#include "xma_parse.h"

using namespace std;

unsigned int Parse_XMA::parse_block(istream& is, long offset, long block_size) {
    long last_offset = offset + block_size;
    unsigned long sample_count = 0;

    for (int packet_number = 0; offset < last_offset; packet_number++) {
        packet_header ph;

        {
            Bit_stream packet_header_stream(is);

            is.seekg(offset);

            packet_header_stream >> ph;
        }

        cout << "Packet #" << packet_number << endl;
        cout << "Frame Count     " << ph.frame_count << endl;
        cout << "Skip Bits       " << ph.skip_bits << endl;
        cout << "Metadata        " << ph.metadata << endl;
        cout << "Packet Skip     " << ph.packet_skip << endl;

        Bit_stream frame_header_stream(is,
                // consecutive
                (packet_size_bytes - packet_header_size_bytes) * 8,
                // skip
                (packet_header_size_bytes + ph.packet_skip * packet_size_bytes) * 8
                );

        // skip initial bits (overflow from a previous packet)
        for (unsigned int i = 0; i < ph.skip_bits; i++) frame_header_stream.get_bit();

        bool packet_end_seen = false;

        for (unsigned int frame_number = 0; frame_number < ph.frame_count; frame_number++) {
            Bit_uint<15> frame_bits;
            frame_header_stream >> frame_bits;

            cout << "   Frame #" << frame_number << endl;
            cout << "   Size " << frame_bits << endl;

            // read frame content (throw away for now)
            unsigned int i;
            // sync?
            {
                Bit_uint<frame_sync_size_bits> sync;
                frame_header_stream >> sync;
                if (sync!= 0x7f00) throw Bad_frame_sync(sync);
            }

            cout << hex;
            const unsigned int frame_body_bits = frame_bits -
                frame_header_size_bits - frame_sync_size_bits - 
                frame_trailer_size_bits;

            for (i = 0; i + 4 <= frame_body_bits; i += 4) {
                Bit_uint<4> nybble;
                frame_header_stream >> nybble;
                cout << nybble;
            }
            cout << " ";
            for (; i < frame_body_bits; i++) {
                cout << (frame_header_stream.get_bit() ? '1' : '0');
            }
            cout << dec << endl;

            // trailer
            {
                Bit_uint<frame_trailer_size_bits> trailer;
                frame_header_stream >> trailer;

                if (trailer == 0)
                {
                    if (frame_number != ph.frame_count-1) throw Early_packet_end();
                    packet_end_seen = true;
                }

                sample_count += samples_per_frame;
            }
        }

        if (!packet_end_seen) throw Missing_packet_end();

        cout << endl;
        offset += (ph.packet_skip + 1) * packet_size_bytes;
    }

    return sample_count;
}
