#include <fstream>
#include <sstream>
#include <stdlib.h>

#include "xma_parse.h"
#include "Bit_stream.h"

using namespace std;

namespace Get_arguments {

struct Argument_error {
    string description;
    Argument_error(string desc) : description(desc) {}
};

void get(int argc, char * argv[], const char ** filename, long *offset, long *block_size);
}

int main(int argc, char * argv[]) {
    const char * file_name;
    long offset, block_size;

    cout << "Experimental XMA parser 0.1 (blame hcs)" << endl << endl << flush;

    try {
        Get_arguments::get(argc, argv, &file_name, &offset, &block_size);
    } catch (Get_arguments::Argument_error ae) {
        cerr << ae.description << endl;
        cerr << "usage: xma_parse filename [-o offset] [-b block size]" << endl;
        exit(EXIT_FAILURE);
    }

    ifstream is(file_name, ios::binary);

    if (!is) {
        cerr << "error opening file!" << endl;
        exit(EXIT_FAILURE);
    }

    cout << "filename: " << file_name << endl;
    cout << "offset: " << hex << offset << dec << endl;
    cout << "block size: " << hex << block_size << dec << endl;
    cout << "------------------" << endl << endl;

    try {
        unsigned long sample_count = Parse_XMA::parse_block(is, offset, block_size);
        cout << endl << sample_count << " samples" << endl;
    }
    catch (const Bit_stream::Out_of_bits& oob) {
        cerr << "error reading bitstream" << endl;
        exit(EXIT_FAILURE);
    }
    catch (const Parse_XMA::Parse_error& pe) {
        cerr << pe << endl;
        exit(EXIT_FAILURE);
    }
}

void Get_arguments::get(int argc, char * argv[], const char ** filename, long *offset, long *block_size) {
    if (argc < 2) {
        throw Argument_error("missing file name");
    }
    *filename = argv[1];
    *offset = 0;
    *block_size = 0x8000;

    for (int argno = 2; argno < argc; argno++) {
        if (argv[argno][0] == '-' && argv[argno][1] != '\0' &&
                argv[argno][2] == '\0') {
            switch (argv[argno][1]) {
                case 'o':
                    if (argno + 1 >= argc) {
                        throw Argument_error("-o needs an argument");
                    }

                    {
                        stringstream argstring(argv[++argno]);
                        argstring >> hex >> *offset;
                        /* TODO: check that conversion went ok */
                    }
                    continue;
                case 'b':
                    if (argno + 1 >= argc) {
                        throw Argument_error("-b needs an argument");
                    }

                    {
                        stringstream argstring(argv[++argno]);
                        argstring >> hex >> *block_size;
                        /* TODO: check that conversion went ok */
                    }
                    continue;
            }
        }

        throw Argument_error(
                string("don't know what to do with argument \"") + argv[argno] + "\"");
    }
}

